#  coding: utf-8
#  Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
################################################################################
import random
from enum import Enum
from typing import List, Dict, Optional, Union

__all__ = ["DeploymentType", "DeviceType", "MessageCode", "HintType", "OrgChangeType", "DeviceChangeType",
           "DeviceGroupChangeType"]


class DeviceChangeType(Enum):
    """
    Enumeration of valid device change types
        See: internal/app/models/enums/devicechangetype/devicechangetype.go

    NOTE: This is the subset from the backend definition that is actually used for hinting
    """
    DEV_GROUP_CHANGE = 301
    DEV_HASH_SAMPLE_REQUEST = 302
    DEV_SENSOR_LOGS_REQUEST = 303
    DEV_HASH_DELETE_REQUEST = 304
    DEV_SENSOR_CERTIFICATE_REQUEST = 306
    DEV_SENSOR_CONFIG_UPDATE = 307
    DEV_SENSOR_TRIGGER_PERFMON = 308
    DEV_SENSOR_KIT_UPDATE = 309
    DEV_SENSOR_ACTION = 310
    DEV_LIVE_QUERY_CHANGE = 317
    DEV_REREGISTER = 321

    @staticmethod
    def all() -> List['DeviceChangeType']:
        """Return list of all DeviceChangeType."""
        return [e for e in DeviceChangeType]

    @staticmethod
    def random() -> List['DeviceChangeType']:
        """Return a list of randomly picked DeviceChangeType"""
        ret = []
        for item in DeviceChangeType.all():
            if random.choice([True, False]):
                ret.append(item)
        return ret


class DeviceGroupChangeType(Enum):
    """
    Enumeration of valid device group change types
        See: internal/app/models/enums/devicegroupchangetype/devicegroupchangetype.go

    NOTE: This is the subset from the backend definition that is actually used for hinting
    """
    DG_POLICY_CHANGE = 201
    DG_LIVE_QUERY_CHANGE = 203

    @staticmethod
    def all() -> List['DeviceGroupChangeType']:
        return [e for e in DeviceGroupChangeType]

    @staticmethod
    def random() -> List['DeviceGroupChangeType']:
        """Return a list of randomly picked DeviceGroupChangeType"""
        ret = []
        for item in DeviceGroupChangeType.all():
            if random.choice([True, False]):
                ret.append(item)
        return ret


class DeviceType(int, Enum):
    """
    Enumeration of valid device types
        See: internal/app/models/enums/devicetype/devicetype.go

    NOTE: "UNKNOWN" is not used
    """
    WINDOWS = 1
    ANDROID = 2
    MAC = 3
    IOS = 4
    LINUX = 5
    OTHER = 10

    @staticmethod
    def all(as_string: bool = False) -> Union[List['DeviceType'], List[str]]:
        if as_string:
            return [e.name for e in DeviceType]
        else:
            return [e for e in DeviceType]

    @staticmethod
    def random() -> List['DeviceType']:
        """Return a list of randomly picked DeviceType"""
        ret = []
        for item in DeviceType.all():
            if random.choice([True, False]):
                ret.append(item)
        return ret


class HintType(Enum):
    """
    Enumeration of valid hint types returned
        See: internal/hints/hinttype/hinttype.go
    """
    NEW_POLICY = 0

    # NOTE: There is no hint type 1

    NEW_CONFIG_DATA = 2
    HASH_LIST_REFRESH = 3
    HASH_LIST_UPLOAD = 4
    CONFER_UPLOAD = 5
    INVALIDATE_NON_WHITELIST_REP = 6
    STATUS_UPDATE = 7
    APC_UPLOAD = 8
    RESUBMIT = 9
    REQUEST_NEWS = 10
    VT_ANALYSIS_UPLOAD = 11
    SOFTWARE_UPGRADE = 12
    SOFTWARE_PATCH = 13
    YARA_AV_SIGNATURES = 14
    YARA_NETWORK_SIGNATURES = 15
    YARA_PE_HEADER_SIGNATURES = 16
    IP_BLOCK_LIST = 17
    IP_FORENSICS_LIST = 18
    HASH_FORENSICS_LIST = 19
    ONETIME_HASH_DELETE_LIST = 20
    PERMANENT_HASH_DELETE_LIST = 21
    SENSOR_CERTIFICATE_UPLOAD = 22
    SENSOR_TRIGGER_PERFMON = 23
    SENSOR_ACTION = 24
    KILL_SWITCH_ON = 25
    ZIP_CONTAINER = 26
    UNINSTALL_CODE = 27
    LIVE_QUERY = 28
    ASYNCREPUTATION = 29
    BLADE_ACTION = 30
    REREGISTER = 31


class MessageCode(Enum):
    """
    Enumeration of valid message codes
        See: internal/hints/messagecode/message_code.go
    """

    UNKNOWN = 0x0000

    REQ_DEVICE_FORENSICS = 0x000B
    RESP_DEVICE_FORENSICS = 0x000C

    REQ_REGISTER_CONTROL = 0x1001
    RESP_REGISTER_CONTROL = 0x1002

    REQ_REGISTRATION_CODE = 0x1003
    RESP_REGISTRATION_CODE = 0x1004

    REQ_DEREGISTER = 0x1017
    RESP_DEREGISTER = 0x1018

    REQ_DOMAIN_CHECK = 0x1019
    RESP_DOMAIN_CHECK = 0x101A

    REQ_VDI_REGISTRATION = 0x101B
    RESP_VDI_REGISTRATION = 0x101C

    REQ_REPUTATION = 0x2005
    RESP_REPUTATION = 0x2006

    REQ_IOS_DEVICE_STATUS = 0x1FFB
    RESP_IOS_DEVICE_STATUS = 0x1FFC

    REQ_ANDROID_DEVICE_STATUS = 0x1FFF
    RESP_ANDROID_DEVICE_STATUS = 0x2000

    REQ_WINOSX_DEVICE_STATUS = 0x2001
    RESP_WINOSX_DEVICE_STATUS = 0x2002

    REQ_POLICY = 0x2003
    RESP_POLICY = 0x2004

    REQ_DEVICE_QUALIFIERS = 0x2009
    RESP_DEVICE_QUALIFIERS = 0x200A

    REQ_CONFIGURATION = 0x200B
    RESP_CONFIGURATION = 0x200C

    REQ_FILE_BEHAVIOR = 0x2010
    RESP_FILE_BEHAVIOR = 0x2011

    REQ_DATA_ACCESS_BEHAVIOR = 0x2012
    RESP_DATA_ACCESS_BEHAVIOR = 0x2013

    REQ_NETWORK_BEHAVIOR = 0x2014
    RESP_NETWORK_BEHAVIOR = 0x2015

    REQ_CREATE_PROCESS_BEHAVIOR = 0x2016
    RESP_CREATE_PROCESS_BEHAVIOR = 0x2017

    REQ_API_CALL_BEHAVIOR = 0x2018
    RESP_API_CALL_BEHAVIOR = 0x2019

    REQ_REGISTRY_ACCESS_BEHAVIOR = 0x201A
    RESP_REGISTRY_ACCESS_BEHAVIOR = 0x201B

    REQ_HELLO = 0x201C
    RESP_HELLO = 0x201D

    REQ_NETWORK_STATUS = 0x201E
    RESP_NETWORK_STATUS = 0x201F

    REQ_ANDROID_APPINFO = 0x2007
    RESP_ANDROID_APPINFO = 0x2008

    REQ_NEWS = 0x2020
    RESP_NEWS = 0x2021

    REQ_NEWSACK = 0x2022
    RESP_NEWSACK = 0x2023

    REQ_ACTION_NOTIFICATION = 0x2024
    RESP_ACTION_NOTIFICATION = 0x2025

    REQ_HASH_EXECUTE_BEHAVIOR = 0x2026
    RESP_HASH_EXECUTE_BEHAVIOR = 0x2027

    REQ_FILE_DELETE_BEHAVIOR = 0x2028
    RESP_FILE_DELETE_BEHAVIOR = 0x2029

    REQ_FILE_UPLOAD = 0x202A
    RESP_FILE_UPLOAD = 0x202B

    REQ_CONFER_FILE_UPLOAD = 0x202C
    RESP_CONFER_FILE_UPLOAD = 0x202D

    REQ_HASH_LIST_REFRESH = 0x202E
    RESP_HASH_LIST_REFRESH = 0x202F

    REQ_GET_HASH_UPLOAD_LIST = 0x2030
    RESP_GET_HASH_UPLOAD_LIST = 0x2031

    REQ_UPLOAD_HASH = 0x2032
    RESP_UPLOAD_HASH = 0x2033

    REQ_GET_SOFTWARE_UPGRADE = 0x2034
    RESP_GET_SOFTWARE_UPGRADE = 0x2035

    REQ_GET_SOFTWARE_PATCH = 0x2036
    RESP_GET_SOFTWARE_PATCH = 0x2037

    REQ_GET_YARA_AV_SIGNATURES = 0x2038
    RESP_GET_YARA_AV_SINGATURES = 0x2039

    REQ_GET_YARA_NETWORK_SIGNATURES = 0x203A
    RESP_GET_YARA_NETWORKAV_SINGATURES = 0x203B

    REQ_GET_YARA_PE_HEADER_SIGNATURES = 0x203C
    RESP_GET_YARA_PE_HEADER_SIGNATURES = 0x203D

    REQ_GET_IP_BLOCK_LIST = 0x203E
    RESP_GET_IP_BLOCK_LIST = 0x203F

    REQ_GET_IP_FORENSICS_LIST = 0x2040
    RESP_GET_IP_FORENSICS_LIST = 0x2041

    REQ_GET_HASH_FORENSICS_LIST = 0x2042
    RESP_GET_HASH_FORENSICS_LIST = 0x2043

    REQ_GET_HASH_DELETE_LIST = 0x2044
    RESP_GET_HASH_DELETE_LIST = 0x2045

    REQ_GET_PERMANENT_HASH_DELETE_LIST = 0x2046
    RESP_GET_PERMANENT_HASH_DELETE_LIST = 0x2047

    REQ_SEND_AD_ASSIGNMENTS = 0x204A
    RESP_SEND_AD_ASSIGNMENTS = 0x204B

    REQ_HASH_UPLOAD_STATUS = 0x204C
    RESP_HASH_UPLOAD_STATUS = 0x204D

    REQ_AV_ANALYSIS_UPLOAD_STATUS = 0x204E
    RESP_AV_ANALYSIS_UPLOAD_STATUS = 0x204F

    REQ_PHISHING_CONFIG = 0x2050
    RESP_PHISHING_CONFIG = 0x2051

    REQ_PHISHING_FILE = 0x2052
    RESP_PHISHING_FILE = 0x2053

    REQ_PHISHING_ALERT_BEHAVIOR = 0x2054
    RESP_PHISHING_ALERT_BEHAVIOR = 0x2055

    REQ_CERTIFICATE_UPLOAD = 0x2056
    RESP_CERTIFICATE_UPLOAD = 0x2057

    REQ_SCAN_ACTION_NOTIFICATION = 0x2058
    RESP_SCAN_ACTION_NOTIFICATION = 0x2059

    REQ_ALARM_NOTIFICATION = 0x2071
    RESP_ALARM_NOTIFICATION = 0x2072

    REQ_CONFIGURATION_V2 = 0x2074
    RESP_CONFIGURATION_V2 = 0x2075

    REQ_SENSOR_ACTION = 0x2076
    RESP_SENSOR_ACTION = 0x2077

    REQ_SENSOR_ACTION_REPORT = 0x2078
    RESP_SENSOR_ACTION_REPORT = 0x2079

    REQ_LIVE_RESPONSE_COMMAND = 0x2080
    RESP_LIVE_RESPONSE_COMMAND = 0x2081

    REQ_RANSOMWARE_PAYLOAD = 0x2090
    RESP_RANSOMWARE_PAYLOAD = 0x2091

    REQ_APC_UPLOAD = 0x20A0
    RESP_APC_UPLOAD = 0x20A1

    REQ_UNINSTALL_CODE = 0x20B0
    RESP_UNINSTALL_CODE = 0x20B1

    REQ_META_DATA_UDPATE_CODE = 0x20B2
    RES_META_DATA_UDPATE_CODE = 0x20B3

    REQ_BULK_BEHAVIOR = 0x20B6
    RESP_BULK_BEHAVIOR = 0x20B7

    # TODO: THESE ARE NOT YET DEFINED IN THE CONFLUENCE DOC!
    REQ_BULK_REPUTATION = 0x20C2
    RESP_BULK_REPUTATION = 0x20C3
    REQ_ASYNC_REPUTATION = 0x20C4
    RESP_ASYNC_REPUTATION = 0x20C5

    REQ_PSCR_EVENT_BATCH = 0x20D0
    RESP_PSCR_EVENT_BATCH = 0x20D1

    # NOTE:  Message Code is being deprecated. REQ_NEW/RESP_NEW are used for backwards compatibility
    REQ_NEW = 0x20D2
    RESP_NEW = 0x20D3


class OrgChangeType(int, Enum):
    """
    Enumeration of valid org change types
        See: internal/app/models/enums/orgchangetype/orgchangetype.go
    """
    ORG_NET_CFG_CHANGE = 101
    ORG_HASH_LIST_CHANGE = 102
    ORG_AV_INTEGRATION_CHANGE = 108
    ORG_KILL_SWITCH_ON = 109
    ORG_UNINSTALL_CODE_CHANGE = 110
    ORG_LIVE_QUERY_CHANGE = 111
    ORG_SET_MELTDOWN_REGISTRY_KEY = 112
    ORG_THREATHUNTER_RULES = 113
    ORG_DISABLE_DEFENSE_RULES = 114
    ORG_BLADE_ACTION_CHANGE = 115

    @staticmethod
    def all(as_value: bool = False) -> Union[List['OrgChangeType'], List[str]]:
        if as_value:
            return [f"{e.value}" for e in OrgChangeType]
        else:
            return [e for e in OrgChangeType]

    @staticmethod
    def random() -> List['OrgChangeType']:
        """Return a list of randomly picked OrgChangeType"""
        ret = []
        for item in OrgChangeType.all():
            if random.choice([True, False]):
                ret.append(item)
        return ret


# NOTE: DeploymentType Must be defined after OrgChangeType due to `all_dict_org()` and `random_dict()`
class DeploymentType(Enum):
    """
    Enumeration of valid deployment types
        See: internal/app/models/enums/deploymenttype

    NOTE: "UNKNOWN" is not used
    """
    ENDPOINT = 1
    WORKLOAD = 2
    VDI = 3
    AWS = 4
    AZURE = 5
    GCP = 6

    @staticmethod
    def all(as_string: bool = False) -> Union[List['DeploymentType'], List[str]]:
        """Return list of all DeploymentType."""
        if as_string:
            return [e.name for e in DeploymentType]
        else:
            return [e for e in DeploymentType]

    @staticmethod
    def all_dict_org() -> Dict['DeploymentType', List[OrgChangeType]]:
        """Return map of all DeploymentType and OrgChangeType."""
        ret = {}
        for item in [e for e in DeploymentType]:
            ret[item] = OrgChangeType.all()
        return ret

    @staticmethod
    def random() -> List['DeploymentType']:
        """Return a list of randomly picked OrgChangeType"""
        ret = []
        for item in DeploymentType.all():
            if random.choice([True, False]):
                ret.append(item)
        return ret

    @staticmethod
    def random_dict() -> Optional[Dict['DeploymentType', List[OrgChangeType]]]:
        """Return a list of randomly picked DeploymentType and OrgChangeType."""
        ret = {}
        for item in DeploymentType.random():
            ret[item] = OrgChangeType.random()
        if len(ret) > 0:
            return ret
        else:
            return None
