package test

import (
	"fmt"
	"net/http"
	"os"
	"strings"
	"testing"

	"github.com/pact-foundation/pact-go/v2/log"
	"github.com/pact-foundation/pact-go/v2/models"
	"github.com/pact-foundation/pact-go/v2/provider"
	"github.com/pact-foundation/pact-go/v2/version"
	"github.com/stretchr/testify/assert"
)

var stateHandlerCalled = false

// create ORG 7ZP4H5XR so we have an org with hints for testing
func setup() error {
	orgChanges := `{"changes":["ORG_NET_CFG_CHANGE",  "ORG_HASH_LIST_CHANGE", "ORG_AV_INTEGRATION_CHANGE", ` +
		`"ORG_KILL_SWITCH_ON", "ORG_UNINSTALL_CODE_CHANGE", "ORG_LIVE_QUERY_CHANGE", "ORG_SET_MELTDOWN_REGISTRY_KEY", ` +
		`"ORG_THREATHUNTER_RULES"]}`

	req, err := http.NewRequest("PUT", "http://127.0.0.1:5004/hinting/v1/orgs/7ZP4H5XR", strings.NewReader(orgChanges))
	if err != nil {
		return err
	}
	resp, err := http.DefaultClient.Do(req)
	if err != nil {
		return err
	}
	if resp != nil && resp.StatusCode != 200 {
		return fmt.Errorf("didnt get 200 from hinting %v", resp.StatusCode)
	}
	return nil
}

func TestV3HTTPProvider(t *testing.T) {
	log.SetLogLevel("DEBUG")
	version.CheckVersion()

	verifier := provider.NewVerifier()

	err := setup()
	if err != nil {
		t.Errorf("Error seting up for provider tests  %v", err)
	}

	isLocal := true
	version := "LOCALDEV"
	branch := "LOCALDEV"
	if sha, ok := os.LookupEnv("CI_COMMIT_SHORT_SHA"); ok {
		version = fmt.Sprintf("1.0.0-SHA%s", sha)
		isLocal = false
	}
	if ciBranch, ok := os.LookupEnv("CI_COMMIT_BRANCH"); ok {
		branch = ciBranch
	}
	// Verify the Provider with broker
	err = verifier.VerifyProvider(t, provider.VerifyRequest{
		ProviderBaseURL: "http://127.0.0.1:5004",
		Provider:        "ems-device-hints",
		ProviderBranch:  branch,
		ProviderVersion: version,
		BrokerURL:       "https://pact-broker.mosaic-dev.sre-tools.carbonblack.io",
		ConsumerVersionSelectors: []provider.Selector{
			&provider.ConsumerVersionSelector{
				MainBranch: true,
			},
		},
		PublishVerificationResults: !isLocal,
		StateHandlers: models.StateHandlers{
			"a healthy service": func(setup bool, s models.ProviderState) (models.ProviderStateResponse, error) {
				stateHandlerCalled = true
				return models.ProviderStateResponse{"uuid": "1234"}, nil
			},
		},
	})

	assert.NoError(t, err)
	assert.True(t, stateHandlerCalled)
}
